# Super Challenge 1: Log Analysis and Reporting Tool

# You are tasked with creating a simplified log analysis tool.  You will be provided with log data
# (simulated here, but could come from a file in a real application).  Your tool should:

# 1. Parse the log data.  Each log entry is a string with the following format:
#    "TIMESTAMP LEVEL: MESSAGE [KEY=VALUE, KEY=VALUE, ...]"
#    - TIMESTAMP: YYYY-MM-DD HH:MM:SS
#    - LEVEL: INFO, WARNING, ERROR, CRITICAL
#    - MESSAGE: A descriptive message.
#    - KEY=VALUE pairs (optional): Additional data in key-value format, separated by commas.

# 2. Store the parsed log entries in a list of dictionaries.  Each dictionary should have keys for:
#    - "timestamp" (datetime object)
#    - "level" (string)
#    - "message" (string)
#    - "attributes" (a dictionary containing the key-value pairs)

# 3. Implement the following functionalities:
#    - `filter_logs(logs, level=None, start_time=None, end_time=None, keyword=None)`:
#      Filters the log entries based on the provided criteria.  All parameters are optional.
#        - `level`: Filters by log level (e.g., "ERROR").
#        - `start_time` and `end_time`: Filters by a time range (datetime objects).
#        - `keyword`: Filters by searching for the keyword in the message (case-insensitive).
#    - `generate_report(logs)`: Generates a summary report of the log data.  The report should include:
#        - The total number of log entries.
#        - The number of entries for each log level.
#        - A list of unique source IPs (if present in the attributes as "source_ip").

# 4. Test your tool with the provided sample log data.  Demonstrate the filtering and reporting functionality.

from datetime import datetime

log_data = [
    "2024-03-15 08:00:00 INFO: User login successful [user=johndoe, source_ip=192.168.1.50]",
    "2024-03-15 08:15:30 WARNING: Disk space low [disk=/dev/sda1, usage=95%]",
    "2024-03-15 08:30:15 ERROR: Database connection failed [source_ip=10.0.0.100]",
    "2024-03-15 09:00:00 INFO: Service started [service=webserver]",
    "2024-03-15 09:45:20 CRITICAL: System crash [source_ip=10.0.0.100, error_code=0xAB12]",
    "2024-03-15 10:15:00 INFO: User logout successful [user=johndoe, source_ip=192.168.1.50]",
    "2024-03-15 10:30:45 ERROR: Invalid request [source_ip=192.168.1.60]",
]

def parse_log_entry(entry):
    # Implement the log parsing logic here.
    pass  # Replace with your code

def filter_logs(logs, level=None, start_time=None, end_time=None, keyword=None):
    # Implement the filtering logic here.
    pass  # Replace with your code

def generate_report(logs):
    # Implement the report generation logic here.
    pass  # Replace with your code

# --- Main program execution ---
parsed_logs = []
for entry in log_data:
  parsed_logs.append(parse_log_entry(entry))

# 1. Show all parsed logs
print("--- All Logs ---")
for log in parsed_logs:
    print(log)

# 2. Demonstrate filtering.
print("\n--- Filtered Logs (ERROR Level) ---")
error_logs = filter_logs(parsed_logs, level="ERROR")
for log in error_logs:
    print(log)

start = datetime(2024, 3, 15, 9, 0, 0)
end = datetime(2024, 3, 15, 10, 0, 0)
print(f"\n--- Filtered Logs (Between {start} and {end}) ---")

time_filtered_logs = filter_logs(parsed_logs, start_time=start, end_time=end)
for log in time_filtered_logs:
  print(log)

print(f"\n--- Filtered Logs (Keyword = 'user') ---")
keyword_filtered_logs = filter_logs(parsed_logs, keyword="user")
for log in keyword_filtered_logs:
  print(log)

# 3. Generate and print the report.
print("\n--- Log Report ---")
report = generate_report(parsed_logs)
print(report)