# Super Challenge 2: Network Packet Analyzer - Solution

packet_data = [
    "192.168.1.1,10.0.0.2,TCP,64,48656c6c6f2c20776f726c6421",  # "Hello, world!" in hex
    "10.0.0.1,192.168.1.2,UDP,128,000102030405060708090a0b0c0d0e0f",
    "192.168.1.1,10.0.0.3,TCP,80,54686973206973206120746573742e",  # "This is a test." in hex
    "10.0.0.4,192.168.1.4,ICMP,56,89abcdef01234567",
    "192.168.1.5,10.0.0.2,TCP,72,416e6f74686572206d6573736167652e",  # "Another message." in hex
]

def parse_packet(packet_string):
    fields = packet_string.split(",")
    source_ip = fields[0]
    destination_ip = fields[1]
    protocol = fields[2]
    length = int(fields[3])
    data = fields[4]

    try:
        data_decoded = bytes.fromhex(data).decode('ascii')
        # Check if the decoded string contains only printable ASCII characters.
        if all(32 <= ord(char) < 127 for char in data_decoded):
            pass #It's all printable
        else:
            data_decoded = "<binary data>"
    except (ValueError, UnicodeDecodeError):
        data_decoded = "<binary data>"

    return {
        "source_ip": source_ip,
        "destination_ip": destination_ip,
        "protocol": protocol,
        "length": length,
        "data": data,
        "data_decoded": data_decoded,
        }

def analyze_packets(packets):
    total_length = 0
    protocol_counts = {}
    destination_ip_counts = {}

    for packet in packets:
        total_length += packet["length"]

        protocol = packet["protocol"]
        protocol_counts[protocol] = protocol_counts.get(protocol, 0) + 1

        destination_ip = packet["destination_ip"]
        destination_ip_counts[destination_ip] = destination_ip_counts.get(destination_ip, 0) + 1

    average_length = total_length / len(packets) if packets else 0

    most_frequent_destination = ""
    max_count = 0
    for ip, count in destination_ip_counts.items():
        if count > max_count:
            most_frequent_destination = ip
            max_count = count

    return {
        "average_length": average_length,
        "protocol_counts": protocol_counts,
        "most_frequent_destination": most_frequent_destination,
    }
def filter_packets(packets, protocol=None, source_ip=None, destination_ip=None):
    filtered = []
    for p in packets:
        if protocol is not None and p['protocol'] != protocol:
            continue
        if source_ip is not None and p['source_ip'] != source_ip:
            continue
        if destination_ip is not None and p['destination_ip'] != destination_ip:
            continue
        filtered.append(p)
    return filtered

# --- Main Program Execution ---

# 1. Parse all packets
parsed_packets = []
for packet in packet_data:
  parsed_packets.append(parse_packet(packet))

# 2. Show parsed packets
print("--- Parsed Packets ---")
for packet in parsed_packets:
    print(packet)
# 3. Analyze
analysis_results = analyze_packets(parsed_packets)
print("\n--- Analysis Results ---")
print(analysis_results)

# 4. Filter and show results
print("\n--- Filtered Packets (TCP) ---")
tcp_packets = filter_packets(parsed_packets, protocol="TCP")
for packet in tcp_packets:
    print(packet)

print("\n--- Filtered Packets (Source IP: 10.0.0.1) ---")
source_ip_packets = filter_packets(parsed_packets, source_ip="10.0.0.1")
for packet in source_ip_packets:
    print(packet)