FILE_NAME = 'data.txt'

def read_data(filename):
    """Reads CSV, returns list of dicts."""
    data = []
    try:
        with open(filename, 'r') as file:
            for line in file:
                name, age_str, score_str = line.strip().split(',')
                try:
                    age = int(age_str)
                    score = int(score_str)
                    data.append({'name': name, 'age': age, 'score': score})
                except ValueError:
                    print(f"Skipping invalid line: {line.strip()}")
    except FileNotFoundError:
        print(f"Error: File '{filename}' not found.")
        return None
    return data


def calculate_average_scores(data):
    """Calculates average score per age."""
    if not data:
        return {}

    age_scores = {}
    age_counts = {}
    for record in data:
        age = record['age']
        score = record['score']
        age_scores[age] = age_scores.get(age, 0) + score
        age_counts[age] = age_counts.get(age, 0) + 1

    average_scores = {
        age: total_score / age_counts[age]
        for age, total_score in age_scores.items()
    }
    return average_scores


def print_results(average_scores):
    """Prints average scores, sorted by age."""
    if not average_scores:
        print("No data.")
        return

    sorted_ages = sorted(average_scores.keys())
    for age in sorted_ages:
        print(f"Avg score for age {age}: {average_scores[age]:.2f}")


def main():
    """Main function."""
    data = read_data(FILE_NAME)
    if data:
        average_scores = calculate_average_scores(data)
        print_results(average_scores)

if __name__ == "__main__":
    main()