# -- ex_1 -- #
def calculate_average(numbers):
    if not numbers:
        return 0
    return sum(numbers) / len(numbers)


# -- ex_2 -- #
def greet_user(name, greeting="Hello"):
    return f"{greeting}, {name}!"


# -- ex_3 -- #
def get_dimensions(length, width):
    area = length * width
    perimeter = 2 * (length + width)
    return area, perimeter


# -- ex_4 -- #
def print_arguments(*args, **kwargs):
    for i, arg in enumerate(args):
        print(f"Positional argument {i}: {arg}")

    for key, value in kwargs.items():
        print(f"Keyword argument {key}: {value}")


# -- ex_5 -- #
def modify_global():
    x = 10

    def inner():
        nonlocal x
        x = 20
        print(f"Inside inner function: x = {x}")

    inner()
    print(f"Inside modify_global after inner call: x = {x}")
    return x


# -- ex_6 -- #
def transform_list(numbers):
    # Filter out odd numbers
    even_numbers = list(filter(lambda x: x % 2 == 0, numbers))

    # Square each number
    squared_numbers = list(map(lambda x: x**2, even_numbers))

    # Sort in descending order
    sorted_numbers = sorted(squared_numbers, reverse=True)

    return sorted_numbers


# -- ex_7 -- #
def calculate_discounted_price(price, discount_percent):
    """Calculates the discounted price of an item.

    Args:
        price (float): The original price of the item.
        discount_percent (float): The discount percentage (0-100).

    Returns:
        float: The discounted price, rounded to 2 decimal places.

    Raises:
        ValueError: If price or discount_percent is negative.
    """
    if price < 0 or discount_percent < 0:
        raise ValueError("Price and discount percentage must be positive values")

    discount_amount = price * (discount_percent / 100)
    discounted_price = price - discount_amount

    return round(discounted_price, 2)


# -- ex_8 -- #
from typing import List, Dict, Optional, Union

def process_data(strings: List[str], uppercase: bool = False) -> Dict[str, int]:
    result = {}

    for string in strings:
        if not string:  # Skip empty strings
            continue

        processed_string = string.upper() if uppercase else string
        result[processed_string] = len(processed_string)

    return result


# -- ex_9 -- #
def apply_operation(func, numbers):
    return [func(num) for num in numbers]

def square(x):
    return x ** 2

def cube(x):
    return x ** 3

def double(x):
    return x * 2


# -- BONUS ex_10 -- #
def counter_factory(step=1):
    count = 0

    def counter(reset=False):
        nonlocal count
        if reset:
            count = 0
            return count
        else:
            count += step
            return count

    return counter


# -- BONUS ex_11 -- #
def validate_person_data(name, age, email, scores):
    # Validate name
    if not isinstance(name, str) or not name:
        raise ValueError("Name must be a non-empty string")

    # Validate age
    if not isinstance(age, int) or age < 0 or age > 120:
        raise ValueError("Age must be an integer between 0 and 120")

    # Validate email
    if not isinstance(email, str) or '@' not in email:
        raise ValueError("Email must be a string containing '@'")

    # Validate scores
    if not isinstance(scores, list) or not scores:
        raise ValueError("Scores must be a non-empty list")

    for score in scores:
        if not isinstance(score, (int, float)) or score < 0 or score > 100:
            raise ValueError("All scores must be numbers between 0 and 100")

    # Return validated data
    return {
        'name': name,
        'age': age,
        'email': email,
        'scores': scores
    }


# -- BONUS ex_12 -- #
def memoize(func):
    cache = {}

    def wrapper(*args):
        if args in cache:
            return cache[args]
        result = func(*args)
        cache[args] = result
        return result

    return wrapper

@memoize
def fibonacci(n):
    if n <= 0:
        return 0
    elif n == 1:
        return 1
    else:
        return fibonacci(n-1) + fibonacci(n-2)