"""
# Exception Handling Practice Challenges

These challenges will help you practice the exception handling concepts
from the lesson. Complete each function according to its description.
"""

"""
Challenge 1: Safe Division

Write a function that safely divides two numbers, handling potential errors:
- If the divisor is zero, return "Error: Division by zero"
- If either input is not a number, return "Error: Invalid input type"
- Otherwise, return the result of the division

Test cases:
- safe_divide(10, 2) → 5.0
- safe_divide(10, 0) → "Error: Division by zero"
- safe_divide("10", 2) → "Error: Invalid input type"
"""
def safe_divide(a, b):
    pass


"""
Challenge 2: File Reader

Write a function that reads the contents of a file and handles potential errors:
- If the file doesn't exist, print "File not found: [filename]" and return None
- If there's a permission error, print "Permission denied for: [filename]" and return None
- If any other exception occurs, print the exception message and return None
- If successful, return the contents of the file

Test case:
- read_file("nonexistent.txt") → Prints "File not found: nonexistent.txt" and returns None
"""
def read_file(filename):
    pass


"""
Challenge 3: Input Parser

Write a function that prompts the user for an integer and handles potential errors:
- The function should continue prompting until valid input is received
- If the user enters non-integer input, inform them and try again
- Add a parameter max_attempts (default: 3) to limit how many times the user can try
- If max attempts are reached, return None
- Otherwise, return the valid integer

Test:
- Call get_integer("Enter a number: ") and try inputs like "abc", "1.5", and "42"
"""
def get_integer(prompt, max_attempts=3):
    pass


"""
Challenge 4: Custom Exception

Create a custom exception class called InvalidAgeError that:
- Inherits from ValueError
- Takes an age parameter
- Has a message that says "Invalid age: [age]. Age must be between 0 and 120."

Then write a function validate_age(age) that:
- Raises TypeError if age is not an integer
- Raises your InvalidAgeError if age is less than 0 or greater than 120
- Otherwise returns the age

Test:
- validate_age(25) → 25
- validate_age(-5) → Raises InvalidAgeError
- validate_age("30") → Raises TypeError
"""
# Define your custom exception
# class InvalidAgeError...

def validate_age(age):
    pass


"""
Challenge 5: Resource Manager

Write a function that demonstrates proper resource management when working with files:
- The function should take a filename and a list of lines to write
- It should use a try/finally structure to ensure the file is closed even if an error occurs
- If an error occurs during writing, print the error and return False
- If successful, return True

Test:
- write_lines("test.txt", ["Line 1", "Line 2"]) → True
"""
def write_lines(filename, lines):
    pass


"""
Challenge 6: Error Logging

Create a more advanced version of the safe_divide function that logs errors:
- Import the logging module at the top of your file
- Configure basic logging with level=logging.INFO
- In your function, log any errors that occur with appropriate severity:
  - Log TypeError with logging.WARNING
  - Log ZeroDivisionError with logging.ERROR
- Return the division result if successful, None otherwise

Test:
- logged_divide(10, 0) → None (and logs error)
"""
import logging
# Configure logging here

def logged_divide(a, b):
    pass


"""
Challenge 7: Retry Mechanism

Write a function that retries an operation multiple times before giving up:
- The function takes a function to execute, max_attempts (default: 3), and a delay (default: 1)
- It should try to execute the function and return its result if successful
- If an exception occurs, it should wait (using time.sleep) for 'delay' seconds and retry
- After 'max_attempts' failures, it should return None or raise the last exception

Test:
- Define a test function that fails a few times, then succeeds
- Call retry(test_function) and verify it eventually succeeds
"""
import time

def retry(operation, max_attempts=3, delay=1):
    pass


"""
Challenge 8: Context Manager

Write a context manager class called Timer that:
- Measures the execution time of a block of code
- Prints the elapsed time when the block exits
- Handles any exceptions that occur in the block by printing "An error occurred: [exception]"
- Re-raises the exception after reporting it

Usage example:
with Timer() as timer:
    # Code to time goes here
    time.sleep(1)
"""
import time

class Timer:
    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass


"""
Challenge 9: Multiple Exception Handling

Write a function that processes a dictionary of values with multiple potential errors:
- The function should take a dictionary and a key to process
- It should try to get the value for the key, convert it to an integer, and divide 100 by it
- Handle KeyError, ValueError, and ZeroDivisionError separately with different messages
- Return a tuple (success_boolean, result_or_error_message)

Test cases:
- process_value({"a": "10"}, "a") → (True, 10.0)
- process_value({"a": "10"}, "b") → (False, "Key not found: b")
- process_value({"a": "abc"}, "a") → (False, "Cannot convert 'abc' to integer")
- process_value({"a": "0"}, "a") → (False, "Cannot divide by zero")
"""
def process_value(data, key):
    pass


"""
Challenge 10: Database Transaction Simulator (Advanced)

Implement a simple database transaction simulator with proper error handling:
- Create a Transaction class that simulates database operations
- Implement methods: start(), execute_query(query), commit(), rollback()
- The execute_query method should randomly fail 30% of the time
- Use try/except/else/finally to ensure proper transaction management
- If any query fails, all changes should be rolled back

Test:
- Run multiple transactions and observe the success/failure patterns
"""
import random

class Transaction:
    def __init__(self):
        self.db = {}
        self.backup = {}
        self.in_transaction = False

    def start(self):
        pass

    def execute_query(self, query):
        pass

    def commit(self):
        pass

    def rollback(self):
        pass

def run_transaction(queries):
    pass


# Testing code - uncomment to test your functions
if __name__ == "__main__":
    # Challenge 1 Test
    print(safe_divide(10, 2))
    print(safe_divide(10, 0))
    print(safe_divide("10", 2))

    # Challenge 2 Test
    print(read_file("nonexistent.txt"))

    # Challenge 3 Test
    # value = get_integer("Enter a number: ")
    # print(f"Valid input: {value}")

    # Challenge 4 Test
    try:
        print(validate_age(25))
        print(validate_age(-5))
    except Exception as e:
        print(f"Caught exception: {e}")

    # Challenge 5 Test
    # print(write_lines("test.txt", ["Line 1", "Line 2"]))

    # Challenge 6 Test
    # print(logged_divide(10, 2))
    # print(logged_divide(10, 0))

    # Challenge 7 Test
    # def test_function():
    #     static_var = getattr(test_function, 'called', 0)
    #     test_function.called = static_var + 1
    #     if test_function.called < 3:
    #         raise ValueError("Not ready yet")
    #     return "Success!"
    #
    # print(retry(test_function))

    # Challenge 8 Test
    # with Timer() as timer:
    #     time.sleep(1)
    #     print("Operation complete")

    # Challenge 9 Test
    # print(process_value({"a": "10"}, "a"))
    # print(process_value({"a": "10"}, "b"))
    # print(process_value({"a": "abc"}, "a"))
    # print(process_value({"a": "0"}, "a"))

    # Challenge 10 Test
    # transaction = run_transaction(["query1", "query2", "query3"])