"""
File System Operations Challenges

This file contains a series of challenges that progressively cover Python's file system operations.
Complete each function according to its docstring.

Topics covered:
- Path management with os.path and pathlib
- Context managers
- Reading and writing files
- Binary files
- Directory operations
- File system navigation
- Error handling
- Walking directories recursively
"""

import os
import shutil
from pathlib import Path


# Challenge 1: Basic Path Operations with os.path
def get_file_info(file_path):
    """
    Use os.path to get information about a file.

    Args:
        file_path (str): Path to a file

    Returns:
        dict: Dictionary containing:
            - 'exists': Boolean indicating if the file exists
            - 'is_file': Boolean indicating if the path is a file
            - 'is_dir': Boolean indicating if the path is a directory
            - 'size': Size of the file in bytes (0 if file doesn't exist)
            - 'basename': Base filename
            - 'dirname': Directory containing the file
            - 'extension': File extension (with the dot)
    """
    # TODO: Implement this function using os.path methods
    pass


# Challenge 2: Path Operations with pathlib
def create_unique_file(directory, base_name, extension):
    """
    Create a unique filename in the specified directory. If a file with
    the given base_name and extension already exists, append a number to
    make it unique (e.g., file.txt, file_1.txt, file_2.txt).

    Use pathlib for all path operations.

    Args:
        directory (str): Directory where the file should be created
        base_name (str): Base name for the file
        extension (str): File extension (without the dot)

    Returns:
        Path: pathlib.Path object representing the unique file path
    """
    # TODO: Implement this function using pathlib
    pass


# Challenge 3: Context Managers
def safe_write_with_backup(file_path, content):
    """
    Write content to a file, but create a backup of the file first if it exists.
    Use context managers for all file operations.

    Args:
        file_path (str): Path to the file
        content (str): Content to write to the file

    Returns:
        tuple: (bool, str) where the boolean indicates success and the string
               contains the backup file path or an error message
    """
    # TODO: Implement this function using context managers
    pass

# CSV Example Data
"""
name,age,city
John,30,New York
Jane,25,Los Angeles
Jim,35,Chicago
Jill,28,San Francisco
"""
# Challenge 4: File Reading and Writing
def csv_to_dict(csv_path, delimiter=','):
    """
    Read a simple CSV file and convert it to a list of dictionaries.
    Assume the first line contains headers.

    Args:
        csv_path (str): Path to the CSV file
        delimiter (str): Delimiter used in the CSV file

    Returns:
        list: List of dictionaries where keys are from the header row
              and values are from each data row
    """




    # TODO: Implement this function
    pass


# Challenge 5: Binary Files
def compare_binary_files(file_path1, file_path2):
    """
    Compare two binary files and return information about differences.

    Args:
        file_path1 (str): Path to the first binary file
        file_path2 (str): Path to the second binary file

    Returns:
        dict: Dictionary containing:
            - 'files_match': Boolean indicating if files are identical
            - 'size_diff': Difference in size (size1 - size2)
            - 'first_diff_pos': Position of first difference (-1 if identical)
            - 'first_diff_bytes': Tuple of differing bytes at first position (None if identical)
    """

    # NOTE: Use (which ls) and (which cp) to get paths to two binary files

    # TODO: Implement this function
    pass


# Challenge 6: Directory Operations
def organize_files_by_extension(source_dir):
    """
Create a tld named files/
in files, create 3 subdirectories:
    - images
    - documents
    - other

    touch the files:
    - image1.jpg
    - image2.png
    - image3.gif
    - document1.txt
    - document2.pdf
    - document3.docx
    - other1.tmp
    - other2.zip
    - other3.tar.gz

    Organize files in the source directory by moving them into subdirectories
    based on their extensions.

    Args:
        source_dir (str): Path to the source directory

    Returns:
        dict: Dictionary mapping extensions to the number of files moved
    """
    # TODO: Implement this function
    pass


# Challenge 7: File System Navigation and Multiple Directory Creation
def create_project_structure(base_dir, project_name):
    """
    Create a project directory structure as follows:
    - project_name/
        - src/
            - __init__.py (empty file)
            - main.py (with content: "print('Hello from {project_name}')")
        - tests/
            - __init__.py (empty file)
            - test_main.py (with content: "# Tests for main.py")
        - docs/
            - README.md (with content: "# {project_name}\n\nProject documentation")

    Use a combination of os and pathlib methods.

    Args:
        base_dir (str): Base directory where the project will be created
        project_name (str): Name of the project

    Returns:
        str: Path to the created project directory
    """
    # TODO: Implement this function
    pass


# Challenge 9: Recursive Directory Walking
def find_duplicate_files(directory):
    """
    Find duplicate files within a directory tree based on file content.

    Args:
        directory (str): Root directory to search

    Returns:
        dict: Dictionary where keys are file content hashes and values are
              lists of paths to files with that content (only include entries
              where there are at least 2 files with the same content)
    """
    # TODO: Implement this function
    pass


# Challenge 10: Comprehensive Challenge
def backup_and_compress(source_dir, backup_dir, exclude_extensions=None):
    """
    Create a backup of a directory structure, excluding files with specified extensions,
    and compress the backup into a zip file with a timestamp.

    Args:
        source_dir (str): Directory to backup
        backup_dir (str): Directory where backup will be stored
        exclude_extensions (list): List of file extensions to exclude (e.g., ['.tmp', '.log'])

    Returns:
        str: Path to the created zip file
    """
    # TODO: Implement this function
    pass


if __name__ == "__main__":
    pass
    # Test code for your challenges
    # Uncomment as you implement each function

    # Challenge 1
    # file_info = get_file_info("./challenges.py")
    # print(f"File info: {file_info}")

    # Challenge 2
    # unique_file = create_unique_file(".", "test", "txt")
    # print(f"Created unique file: {unique_file}")

    # Continue with tests for other challenges...