# Nmap Scan Results Parser

## Assignment Overview

For this assignment, you will create a Python script that parses network scan results from the Nmap security tool and generates a summary report. This will help you practice string manipulation, data analysis, and formatting output in Python.

## What is Nmap?

Nmap (Network Mapper) is a popular open-source tool used for network discovery and security auditing. It scans networks and outputs information about hosts and the services running on them. Security professionals use this information to identify potential vulnerabilities or monitor their networks.

## Task Description

Your Python script should:

1. Read Nmap scan results from a file
2. Parse the results to extract information about hosts and open ports
3. Calculate and output the following statistics:
   - Total number of unique ports found open across all hosts
   - Total number of unique hosts found with at least one open port
   - Average number of open ports per host (rounded to the nearest integer)
4. For each unique port found, output:
   - Port number
   - Service name
   - Count (number of hosts where this port is open)
5. Format the output exactly as shown in the example below

## Input Format

The input will be standard Nmap scan results with multiple host scan reports. Each report includes an IP address, latency information, and a list of open ports with their service names.

## Expected Output Format

Your script should produce output in the following format:

```
### --- Number of unique ports found open: X
### --- Number of unique hosts found with open ports: Y
### --- Average number of open ports rounded to the nearest integer: Z

### --- Port : 80 Service : http --- ###
Port Number: 80
Service Name: http
Count: A

### --- Port : 443 Service : https --- ###
Port Number: 443
Service Name: https
Count: B

### --- Port : 8080 Service : http-proxy --- ###
Port Number: 8080
Service Name: http-proxy
Count: C
...
```

## Requirements

- The port listings should be sorted by count (highest first) and then by port number (for ports with the same count)
- You should only use Python's standard library (no external libraries)
- Follow all PEP8/Google Python Style Guide recommendations learned to this point

## Running the Script

Your script should accept a filename as a command-line argument:

```
python main.py scan_results.txt
```

Where `scan_results.txt` is a file containing Nmap scan results in the format shown in the example.
