# Loops

## Objectives

- Loops
    - for loops
    - enumerate
    - while loops
    - Loop Control (break, continue)
    - Loop with else
    - Comprehensions (Basic)


## Loops

Loops allow you to repeat code multiple times.

### for loops

The for loop is used to iterate over sequences:

!!! Note "Sequences"
    Remember that the term "sequence" is used to describe a list, tuple, string, or custom data structure that can be iterated over. What determines whether an object can be iterated over will be covered in a later section.

```python

# String
for char in "hello":
    print(char)

# Tuple
my_tuple = (1, 2, 3)
for num in my_tuple:
    print(num)

# Dictionary (keys)
my_dict = {"a": 1, "b": 2}
for key in my_dict:  # Iterates over keys by default
    print(key)

# Dictionary (values)
for value in my_dict.values():
    print(value)

# Dictionary (key-value pairs)
for key, value in my_dict.items():
    print(f"Key: {key}, Value: {value}")

# Iterating over a range
for i in range(3):
    print(i)  # Outputs: 0, 1, 2

# Iterating over a list
fruits = ["apple", "banana", "cherry"]
for fruit in fruits:
    print(fruit)

# NOTE: how easy it is to iterate over a list without specifying an index, counter, or length as in other languages.

# Enumerate for index and value
for index, fruit in enumerate(fruits):
    print(f"{index}: {fruit}")
```

### enumerate

#TODO: Add enumerate example

- `enumerate(iterable, start=0)`
- The enumerate() function takes an iterable as an argument and generates tuples of the form (index, item)

```python
# Pythonic Traditional Way
fruits = ["orange", "apple", "mango", "lemon"]

for index in range(len(fruits)):
    fruit = fruits[index]
    print(index, fruit)
# 0 orange
# 1 apple
# 2 mango
# 3 lemon
```

```python
# Using enumerate
for index, fruit in enumerate(fruits):
    print(index, fruit)
# 0 orange
# 1 apple
# 2 mango
# 3 lemon
```

```python
# Loop in in Parallel
names = ["John", "Jane", "Jim", "Jill"]
ages = [25, 30, 35, 40]

for name, age in zip(names, ages): # zip() takes multiple iterables and returns an iterator of tuples
    print(f"{name} is {age} years old")

# Output:
# John is 25 years old
# Jane is 30 years old
# Jim is 35 years old
# Jill is 40 years old
```

```python
from itertools import chain
# Loop in Sequence
first = [1, 2, 3]
second = [4, 5, 6]
third = [7, 8, 9]

# itertools.chain() takes multiple iterables and returns an
# iterator that yields elements from the first iterable until it is exhausted,
# then proceeds to the next iterable, and continues until all iterables are exhausted.
for value in chain(first, second, third):
    print(value**2)

# Output:
# 1
# 4
# 9
# 16
# 25
# 36
# ...
```

### while loops

While loops repeat as long as a condition is True:

!!! Warning "DANGEROUS - Infinite loop!"
    ```python
    while True:
        print("This will run forever!")
    ```

```python
# Always True loop
while game_running == True:
    # game loop code
    show_game_screen()
    get_mouse_input()
    update_game_state() # One of these functions will typically set game_running to False

# Counting loop
count = 0
while count < 3:
    print(count)
    count += 1

# With user input
password = ""
while password != "secret":
    password = input("Enter password: ")
```

### Loop Control

Break and continue help control loop execution:

```python
# break example
for i in range(10):
    if i == 5:
        break  # Exit loop when i is 5
    print(i)

# NOTE: Every line below the break, but inside the outer loop block will be skipped
# and no further loops will be executed.

# continue example
for i in range(5):
    if i == 2:
        continue  # Skip printing 2
    print(i)

# NOTE: continue will skip the rest of the loop body like break,
# but will continue to execute the next iteration.
```

### Loop with else

Python allows an else clause with loops that executes when the loop completes normally:

```python
# Finding a number
numbers = [1, 3, 5, 7, 9]
search_for = 4

for num in numbers:
    if num == search_for:
        print("Found it!")
        break
else:
    print("Number not found")  # This will execute

# NOTE: The else clause will execute if the loop reaches the end of the sequence without breaking.
```

### Comprehensions

List and Dictionary comprehensions provide a concise way to create lists and dictionaries:

!!! Warning "New Developer Note"

    **Warning 1:** Remeber that the return value of a comprehension is a new list or dictionary. If you are just trying to compress a for loop into a single line or something else weird, just use a regular for loop.

    **Warning 2:** Novice programmers often "fall in love" with comprehensions and use them for everything. This is a bad idea. Comprehensions are a powerful tool and should be used when it makes sense to do so.

    **Warning 3:** KISS (Keep It Simple Stupid). If you think a novice programmer would struggle to understand the comprehension, then be assured that experienced programmers will to. There are plenty of professional developers who strictly prohibit the use of comprehensions, believing that they make code harder to read and understand.

**List Comprehensions**

```python
# Traditional way
squares = []
for x in range(5):
    squares.append(x**2)

# Using list comprehension
squares = [x**2 for x in range(5)]
print(squares)  # Output: [0, 1, 4, 9, 16]

# With conditional
even_squares = [x**2 for x in range(10) if x % 2 == 0]
print(even_squares)  # Output: [0, 4, 16, 36, 64]
```

**Dictionary Comprehensions**

```python
# Traditional way
squares = {}
for x in range(5):
    squares[x] = x**2

# Using dictionary comprehension
squares = {x: x**2 for x in range(5)}
print(squares)  # Output: {0: 0, 1: 1, 2: 4, 3: 9, 4: 16}

# NOTE: The main difference is that 2 expressions are used to the left of the for loop instead of 1. The first is the key, the second is the value.
```

## Check on Learning / Challenges

!!! Warning "Complete challenges on your VTA workstation and dev environment."

- Today's challenges can be found at `~/170D/cybbh/challenges/py_1_1/06_control_flow/{challenge_number}`
- All challenge prompts are contained in their respective `main.py` file as comments.
- The `solution.py` file exists primarily as a last resort after you've exhausted getting help from the instructor, a classmate, or the web.
