# ==================== Challenge 1 Solution ====================

# 1. Create variables to store device information
hostname = "router-01"
ip_address = "192.168.1.1"
model = "Cisco ISR4331"
is_enabled = True
services = ["ssh", "http", "snmp"]

# 2. Print variables using f-strings
print(f"Hostname: {hostname}")
print(f"IP Address: {ip_address}")
print(f"Model: {model}")
print(f"Is Enabled: {is_enabled}")
print(f"Services: {services}")


# ==================== Challenge 2 Solution ====================

# 1. Create a list called devices
devices = []

# 2. Create three device dictionaries
device1 = {
    "hostname": "router-01",
    "ip_address": "192.168.1.1",
    "model": "Cisco ISR4331",
    "is_enabled": True,
    "services": ["ssh", "http", "snmp"]
}

device2 = {
    "hostname": "switch-02",
    "ip_address": "192.168.1.2",
    "model": "Cisco Catalyst 9300",
    "is_enabled": True,
    "services": ["ssh", "https"]
}

device3 = {
    "hostname": "firewall-01",
    "ip_address": "10.0.0.1",
    "model": "Palo Alto PA-3220",
    "is_enabled": False,
    "services": []  # No services running
}

# 3. Append dictionaries to the list
devices.append(device1)
devices.append(device2)
devices.append(device3)

# 4. Iterate and print with a for loop
for device in devices:
    print(f"Device: {device['hostname']}, IP: {device['ip_address']}")


# ==================== Challenge 3 Solution ====================

# Create a dummy devices.txt file (for demonstration purposes within the code)
# In a real scenario, this file would already exist.
with open("devices.txt", "w") as f:
    f.write("router-01,192.168.1.1,Cisco ISR4331,True,ssh;http;snmp\n")
    f.write("switch-02,192.168.1.2,Cisco Catalyst 9300,True,ssh;https\n")
    f.write("firewall-01,10.0.0.1,Palo Alto PA-3220,False,\n")
    f.write("router-04,10.0.0.4,Juniper SRX300,True,ssh\n") #added a 4th for additional testing


devices = []

# 3. Read data from file
with open("devices.txt", "r") as f:
    for line in f:
        # Split the line into fields
        fields = line.strip().split(",")

        # Create the device dictionary
        device = {
            "hostname": fields[0],
            "ip_address": fields[1],
            "model": fields[2],
            "is_enabled": fields[3] == "True",  # Convert to boolean
            "services": fields[4].split(";") if fields[4] else []  # Split services, handle empty string
        }

        # Append to the list
        devices.append(device)

# 4. Print the contents of the devices list
for device in devices:
    print(device)


# ==================== Challenge 4 Solution ====================

# (Assuming the 'devices' list from Challenge 3 is available)

# Re-create the 'devices' list for standalone execution of this challenge
devices = []
with open("devices.txt", "r") as f:
    for line in f:
        fields = line.strip().split(",")
        device = {
            "hostname": fields[0],
            "ip_address": fields[1],
            "model": fields[2],
            "is_enabled": fields[3] == "True",
            "services": fields[4].split(";") if fields[4] else []
        }
        devices.append(device)


# 1. check_ip_address_format function
def check_ip_address_format(device):
    ip = device["ip_address"]
    parts = ip.split(".")
    if len(parts) != 4:
        return False
    for part in parts:
        if not part.isdigit():
            return False
        num = int(part)
        if num < 0 or num > 255:
            return False
    return True

# 2. check_device_enabled function
def check_device_enabled(device):
    return device["is_enabled"]

# 3. check_service_running function
def check_service_running(device, service_name):
    return service_name in device["services"]

# 4. Test the functions
test_device = devices[0]  # Use the first device for testing

print(f"IP Address Check ({test_device['hostname']}): {check_ip_address_format(test_device)}")
print(f"Device Enabled Check ({test_device['hostname']}): {check_device_enabled(test_device)}")
print(f"Service 'ssh' Running ({test_device['hostname']}): {check_service_running(test_device, 'ssh')}")
print(f"Service 'telnet' Running ({test_device['hostname']}): {check_service_running(test_device, 'telnet')}")


# ==================== Challenge 5 Solution ====================

# (Assuming the 'devices' list and functions from Challenge 4 are available)

# Re-create the 'devices' list for standalone execution
devices = []
with open("devices.txt", "r") as f:
    for line in f:
        fields = line.strip().split(",")
        device = {
            "hostname": fields[0],
            "ip_address": fields[1],
            "model": fields[2],
            "is_enabled": fields[3] == "True",
            "services": fields[4].split(";") if fields[4] else []
        }
        devices.append(device)

# Re-create the functions from Challenge 4
def check_ip_address_format(device):
    ip = device["ip_address"]
    parts = ip.split(".")
    if len(parts) != 4:
        return False
    for part in parts:
        if not part.isdigit():
            return False
        num = int(part)
        if num < 0 or num > 255:
            return False
    return True

def check_device_enabled(device):
    return device["is_enabled"]

def check_service_running(device, service_name):
    return service_name in device["services"]

# 1. audit_devices function
def audit_devices(devices, disallowed_services):
    for device in devices:
        print(f"--- Auditing device: {device['hostname']} ---")

        ip_valid = check_ip_address_format(device)
        print(f"  IP Address Valid: {ip_valid}")

        enabled = check_device_enabled(device)
        print(f"  Device Enabled: {enabled}")

        disallowed_found = []
        for service in disallowed_services:
            if check_service_running(device, service):
                disallowed_found.append(service)
        if disallowed_found:
            print(f"  Disallowed Services Found: {', '.join(disallowed_found)}")
        else:
            print("  No disallowed services found.")
        print()  # Add an empty line for better readability

# 2. Call audit_devices
disallowed_services = ["telnet", "ftp"]
audit_devices(devices, disallowed_services)


# ==================== Challenge 6 Solution ====================

# (Assuming the 'devices' list and functions from previous challenges are available)


devices = []
with open("devices.txt", "r") as f:
    for line in f:
        fields = line.strip().split(",")
        device = {
            "hostname": fields[0],
            "ip_address": fields[1],
            "model": fields[2],
            "is_enabled": fields[3] == "True",
            "services": fields[4].split(";") if fields[4] else []
        }
        devices.append(device)

def check_ip_address_format(device):
    ip = device["ip_address"]
    parts = ip.split(".")
    if len(parts) != 4:
        return False
    for part in parts:
        if not part.isdigit():
            return False
        num = int(part)
        if num < 0 or num > 255:
            return False
    return True

def check_device_enabled(device):
    return device["is_enabled"]

def check_service_running(device, service_name):
    return service_name in device["services"]

# 1. Modified audit_devices function (with output_file)
def audit_devices(devices, disallowed_services, output_file):
    with open(output_file, "w") as f:  # Open for writing
        for device in devices:
            f.write(f"--- Auditing device: {device['hostname']} ---\n")

            ip_valid = check_ip_address_format(device)
            f.write(f"  IP Address Valid: {ip_valid}\n")

            enabled = check_device_enabled(device)
            f.write(f"  Device Enabled: {enabled}\n")

            disallowed_found = []
            for service in disallowed_services:
                if check_service_running(device, service):
                    disallowed_found.append(service)
            if disallowed_found:
                f.write(f"  Disallowed Services Found: {', '.join(disallowed_found)}\n")
            else:
                f.write("  No disallowed services found.\n")
            f.write("\n")  # Add an empty line

# 2. Call audit_devices with output file
disallowed_services = ["telnet", "ftp"]
audit_devices(devices, disallowed_services, "audit_report.txt")

# The audit_report.txt file will be created/overwritten with the report.

