Okay, here's a comprehensive review document with questions covering the material you provided, formatted in Markdown, along with some "bad code" examples at the end.

# Programming Fundamentals and Python Review

## Programming Fundamentals

### What is Programming?

1. What is the role of "data" in a program? Give an example.
2. Describe, in your own words, what programming is.

### Sequential Instruction

1. Describe the concept of sequential instruction execution. Why is the order of instructions important?
2. How does Python handle variable creation and assignment? How is this different from languages like C++ that require explicit variable declarations?
3. Explain how sequential execution works in the following code snippet:

    ```python
    x = 10
    y = x + 5
    print(y)
    ```

### Programming Languages

1. Why are there different programming languages?  What are some advantages of Python's syntax?
2. Compare and contrast the Python and C++ code examples in the lesson. What differences do you notice?
3. What are some of the tradeoffs between using a high-level language like Python versus a lower-level language like C++?

### Compiled vs Interpreted Languages

1. What is the key difference between compiled and interpreted languages?
2. Is Python a compiled or interpreted language? What are the implications of this?
3. List two advantages and two disadvantages of interpreted languages compared to compiled languages.
4. How does the concept of "platform independence" relate to interpreted languages?
5. Is speed an absolute concept? How does it relate to programming languages?

### Code and Data

1. Explain the relationship between "code" and "data" in a program.
2. Give examples of different data types in Python.
3. What is the significance of data in program design? Why is it important to consider the type of data you're working with?
4. What's the difference between a list and a dictionary in Python, in terms of how they store data?
5. Explain what the term "data-oriented design" might mean.

### Basic Syntax Concepts

1. What is the role of indentation in Python? How does it differ from other languages like C++ or Java?
2. What are comments used for in code? Provide examples of both single-line and multi-line comments.
3. List three arithmetic operators and three comparison operators in Python, with examples.
4. Explain the following terms, and give an example of each:
    * Literal
    * Keyword
    * Object
    * Function
    * Variable
    * Argument
    * Return

## Python Ecosystem

### Python Installation and Versions

1. What command would you use to check the version of Python installed on your system?
2. What is the latest version of Python 3? Why is it important to use Python 3 instead of Python 2?
3. Where would you find the most accurate documentation for Python?

### Python Execution Methods

1. Describe two different ways you can execute Python code.
2. What is the REPL, and how can it be useful for testing code?
3. What is the `-c` flag used for when running Python from the command line?
4. Is the REPL good for testing large programs?

### Python Documentation

1. How can you access documentation for a specific Python function or module within the REPL?
2. What is the purpose of the `dir()` function? How might it be used to explore a module?
3. How could you find out what methods are available for a Python list object?

### The Standard Library

1. What is the Python Standard Library? Give three examples of modules it contains.
2. What are some common tasks that the Standard Library can help you with?  Give examples using specific modules.
3. Why is it called the "Standard Library"?

### PyPI and Third-Party Packages

1. What is PyPI? What is its purpose?
2. Name three popular third-party Python packages and briefly describe their purpose.
3. What is the difference between a "library package" and a "CLI tool" in the Python ecosystem?
4. How can you usually tell whether a package is intended to be used as a library or a CLI tool?

### Virtual Environments (venv)

1. What is a virtual environment, and why is it important to use them in Python development?
2. Describe the steps to create and activate a virtual environment using `venv`.
3. How do virtual environments help prevent dependency conflicts between different projects?
4. What command deactivates a virtual environment?

### Package Management (pip)

1. What is `pip`, and what is it used for?
2. How would you install a specific version of a package using `pip`?
3. What is a `requirements.txt` file, and what is its purpose?
4. What command would you use to generate a `requirements.txt` file from your current environment?
5. What is the purpose of the `-r` flag when using `pip install`?
6. Name 3 other package managers besides `pip`.
7. Explain the difference between `application` and `development` dependencies.

### Terminology
1. Explain each of the following terms in your own words and provide a Python code example where applicable:

    *   Literal
    *   Comments
    *   Keywords
    *   Objects
    *   Modules
    *   Packages
    *   Functions
    *   Variables
    *   Expressions
    *   Statements
    *   Arguments
    *   Parameters
    *   Return
    *   Docstrings

## Python Basics

### Python Style Guide and PEPs

1. What does PEP stand for? What is the purpose of PEPs?
2. Which PEP provides the official style guide for Python code?
3. What is the "Zen of Python"?  How can you view it?  What are three principles from it that resonate with you?
4. Why is consistent code style important in a programming project?
5. What are the recommended naming conventions for variables, functions, and classes in Python?
6. Why should you avoid using single-letter variable names (except for simple loop counters)?
7. Why is it bad practice to use built-in function names (like `list` or `str`) as variable names?

### Variables and Assignment

1. What is a variable in Python? How is it different from a variable in mathematics?
2. Explain the concept of "multiple assignment" in Python.
3. What is "augmented assignment"? Provide three examples.
4. What does it mean for a variable to be "reassigned" in Python?

### Dynamic vs Static Typing

1. Explain the difference between static and dynamic typing.
2. Is Python statically or dynamically typed? What are the advantages and disadvantages of this?
3. What is "type inference"? How does it work in Python?
4. What is "type checking"? When does it occur in Python?
5. Can a variable change its type during the execution of a Python program? Explain with an example.
6. What is type hinting?

### Basic Data Types Overview
1. What is a `type` in Python?
2. What are `built-in` types?
3. What's the difference between an `int` and a `float`?
4. What is "duck typing"?

### Number Data Types

1. What are the main numeric types in Python?
2. What are the limitations of `float` values?  Why do they sometimes produce unexpected results?
3. What is the `Decimal` type, and when would you use it instead of `float`?
4. What is the `Fraction` type, and when would you use it?
5. What does the acronym "Flops" stand for, and how does it relate to the hardware?
6. When would you choose to use `float` values, and when would you choose `Decimal`?
7. What's the difference between `sys.getsizeof(3.14)` and `sys.getsizeof(Decimal('3.14'))`?

### Boolean Data Types

1. What are the two possible values of a boolean variable?
2. What is "truthiness" in Python?  Give examples of values that are considered "truthy" and "falsy".
3. What are the three main boolean operators, and how do they work?
4. Explain "short-circuit evaluation" in the context of boolean operators.
5. How would you make sure the expression `y/x` is not evaluated if x is 0?

### Strings (Basic)

1. What is a string in Python?  How can you create a string literal?
2. How can you concatenate strings?
3. Name 3 `string` methods

### Type Conversion

1. What is type conversion? Why is it sometimes necessary?
2. How can you convert a string to an integer? What happens if the string doesn't represent a valid integer?
3. What is the result of `bool("False")`?  Why?
4. How can you convert a number to its binary or hexadecimal representation?

### Operators

1. List and describe the arithmetic operators in Python.
2. What is the difference between `/` and `//`?
3. What is the modulus operator, and what does it do?
4. What are chained comparisons? Give an example.
5. What is the difference between `==` and `is`?
6. Describe bitwise operators and give 3 examples of when they are used.

### Python Keywords

1. What are Python keywords?
2. Why can't you use keywords as variable names?
3. Name five Python keywords.

## Strings and IO

### Strings

1. How can you create multi-line strings in Python?
2. What is string slicing?  Explain with examples how to extract substrings using slicing.
3. Explain the parameters of the slice operator: `[start:stop:step]`.
4. What does it mean that strings are immutable?
5. Describe at least five common string methods and what they do.
6. Explain the difference between the following string formatting methods:
    *   `%` - formatting
    *   `str.format()`
    *   f-strings
7. What are escape sequences in strings? Give three examples.
8. What are raw strings, and when are they useful?
9. What is Unicode? How does it relate to strings in Python?
10. What is the difference between `ord('A')` and `chr(65)`?
11. What is the difference between a string and a bytes object?  When would you use each?
12. What are some common encoding schemes used in Python?
13. How can you handle encoding errors when working with strings and bytes?
14. What is the difference between a `bytes` object and a `bytearray` object?
15. What module can you import for creating hashes?

### Input/Output

1. How can you print output to the console in Python?
2. How can you control the separator and end characters when using `print()`?
3. How can you read user input from the console?
4. How can you convert user input to a specific data type (e.g., integer or float)?
5. What is the recommended way to open and work with files in Python?  Why?
6. Explain the different file opening modes (`'r'`, `'w'`, `'a'`, `'rb'`, `'wb'`, etc.).
7. How can you read a file line by line?
8. How can you write to a file?
9. How can you append to a file?
10. How can you handle multiple files within a single `with` statement?
11. What are some best practices for file handling in Python?
12. What are some common pitfalls to avoid when working with files?
13. Explain the difference between `os` and `pathlib`

## Data Structures

### Lists

1. What is a list in Python?  How is it different from an array in other languages (like C++)?
2. How can you create a list?
3. How do you access elements in a list?  What is the index of the first element?
4. How can you modify a list (add, remove, change elements)?
5. What is list slicing?  Give examples of how to use it.
6. What is a nested list?  How can you access elements in a nested list?
7. List five common list methods and describe what they do.
8. How can you concatenate lists?
9. How can you check if an element is present in a list?
10. How can you sort a list? What's the difference between `sorted()` and `.sort()`?
11. What is a list view?

### Dictionaries

1. What is a dictionary in Python?
2. How is a dictionary different from a list?
3. What are keys and values in a dictionary?  What are the restrictions on keys?
4. How can you create a dictionary?
5. How can you access, add, and modify values in a dictionary?
6. What happens if you try to access a key that doesn't exist in a dictionary using square brackets?  How can you avoid this?
7. List five common dictionary methods and describe what they do.
8. How can you iterate through a dictionary (keys, values, or key-value pairs)?
9. What is a dictionary view?

### Basic Collection Operations

1. How can you sort a list of dictionaries based on a specific key?
2. How would you implement basic network log analysis using lists and dictionaries?

### Real-World Examples
1. How could you use Python data structures to represent and analyze:
   * Network security log entries?
   * Port scanner results?
   * Network packet data?
   * System user information?
   * Firewall rules?

## Control Flow

### Conditional Statements

1. What is the purpose of conditional statements in programming?
2. Explain the syntax of `if`, `elif`, and `else` statements in Python.
3. What are the comparison operators in Python?
4. What are the logical operators in Python?
5. What is a ternary operator? When should you use it, and when should you avoid it?
6. How can you combine logical operators and ternary operators in a single expression?
7. How can you use the `pass` keyword in a conditional statement?

### Loops

1. What is the purpose of loops in programming?
2. What are the two main types of loops in Python?
3. Explain the syntax of a `for` loop. What types of objects can you iterate over with a `for` loop?
4. How can you access both the index and value of an item while iterating through a list?
5. Explain the syntax of a `while` loop.
6. What is an infinite loop? How can you create one, and how can you avoid them?
7. What are `break` and `continue` statements? How do they affect loop execution?
8. What is the purpose of the `else` clause in a loop? When does it execute?
9. What are list comprehensions? Give an example of how to create a list using a list comprehension.
10. What are dictionary comprehensions?  Give an example.
11. When should you use comprehensions, and when should you avoid them?
12. What is the return value of a comprehension?

## Bad Code Examples (For Class Correction)

Here are some code examples with deliberate errors or bad practices. The class should identify the problems and suggest corrections.

**Example 1:  Variable Naming and Style**

```python
# Whats wrong with this code?
A = 10
b = "Hello"
My_variable = [1, 2, 3]

def Calculate_sum(x,Y):
   return x   +Y

print(Calculate_sum (  A, 5))
```

**Example 2: Type Errors and Conversion**

```python
# Whats wrong with this code?
user_input = input("Enter a number: ")
result = user_input * 2
print("The result is: " + result)
```

**Example 3: File Handling**

```python
# Whats wrong with this code?
file = open("my_file.txt", "w")
file.write("Some data")
print(file.read()) # Try and read from a file opened in write mode

# forgot to close the file, or didn't open with a context manager

# Whats wrong with this code?
my_list = [1, 2, 3]
my_string = my_list[3]
```

**Example 4: List Indexing**

```python
# Whats wrong with this code?
my_list = [1, 2, 3]
my_string = my_list[3]

```

**Example 5: Dictionary Access**

```python
# Whats wrong with this code?
my_dict = {"name": "Alice", "age": 30}
print(my_dict["city"]) # KeyError: 'city'
```

**Example 6: Infinite Loop**

```python
# Whats wrong with this code?
x = 10
while x > 0:
    print(x)
# missing x -= 1
```

**Example 7: Overly Complex Comprehension**

```python
# Whats wrong with this code?
numbers = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
result = [x**2 if x % 2 == 0 else x * 3 for x in numbers if x > 5 and x < 9]
# should just be a for loop
```

**Example 8:  Ignoring `try-except` with file operations**

```python
file_path = input("Enter the file path: ")

# Opens file, but doesn't handle any errors like:
# - FileNotFoundError
# - PermissionError
# - IsADirectoryError
# - UnicodeDecodeError
f = open(file_path, 'r')
contents = f.read()
print(contents)
f.close() # what if the file doesn't close?
```

**Example 9: Mutable Default Arguments (Advanced - Might not be covered yet)**

```python
# Whats wrong with this code?
def add_to_list(item, my_list=[]):
    my_list.append(item)
    return my_list

list1 = add_to_list(1)
list2 = add_to_list(2)
print(list1)  # Expected: [1], Actual: [1, 2]
print(list2)  # Expected: [2], Actual: [1, 2]
```

These review questions and bad code examples cover a wide range of topics from the provided lessons. They are designed to encourage critical thinking and problem-solving skills. The bad code examples highlight common mistakes and bad practices, allowing students to practice identifying and correcting errors. This comprehensive review should solidify understanding of the core Python concepts presented.
