### Debugging Techniques

When exceptions occur, you need effective debugging techniques to identify the root cause. Here are some key approaches:

-   **Print Statements:** Strategically placed `print()` statements can help you trace the execution flow and inspect variable values.

-   **`pdb` (Python Debugger):** The built-in Python debugger allows you to step through your code line by line, set breakpoints, inspect variables, and evaluate expressions.

    ```python
    import pdb

    def my_function(x, y):
        result = x + y
        pdb.set_trace()  # Set a breakpoint here
        result = result * 2
        return result


    my_function(5, 3)
    ```

    When the program hits `pdb.set_trace()`, it enters the debugger. Common `pdb` commands include:

    -   `n` (next): Execute the next line of code.
    -   `s` (step): Step into a function call.
    -   `c` (continue): Continue execution until the next breakpoint.
    -   `p <expression>` (print): Evaluate and print an expression.
    -   `l` (list): List the source code around the current line.
    -   `b <line number>` (break): Set a breakpoint at a specific line number.
    -   `q` (quit): Quit the debugger.

-   **IDE Debuggers:** Most integrated development environments (IDEs) like VS Code, PyCharm, and others provide powerful graphical debuggers with features like breakpoints, variable inspection, call stack visualization, and more.

-   **Traceback Analysis:** The traceback (the error message you see when an exception is unhandled) provides valuable information, including:
    -   The type of exception.
    -   The line number where the exception occurred.
    -   The call stack (the sequence of function calls that led to the error).
    -   Stack Trace: The call stack is the stack of functions that were called up to that point. Python lists the call stack as a **stack trace**
    -   The call stack is very useful because it does not only show where the error happened, but also how the program ended up in that place of the code.
