## Classes Review

### 1. What are the main components of a Python class?

- Class definition: This is the blueprint for creating objects. It's defined using the `class` keyword followed by the class name.
- Attributes: These are the data members of the class. They represent the state or properties of an object. Attributes can be defined within the class or within its methods.
- Methods: These are functions defined within the class that describe the behaviors of the objects. They can manipulate the object's attributes and perform operations.
- Constructor: The `__init__()` method is a special method that initializes a new object when it's created.
- Self parameter: This is used in method definitions to refer to the instance of the class.
- Class variables: These are variables shared by all instances of the class.
- Instance variables: These are variables unique to each instance of the class. (self.attribute)
- Special (dunder) methods: These are methods with double underscores that provide special behaviors to the class.

### 2. What's the purpose of the self parameter in class methods?

  - Instance reference: It refers to the specific instance of the class that the method is being called on. This allows the method to access and modify the instance's attributes.
  - Distinguishing between instances: It helps Python distinguish between different instances of the same class, ensuring that methods operate on the correct object.
  - Accessing instance variables: Through `self`, methods can access and modify instance variables, allowing for state management within the object.
  - Method chaining: It allows methods to return `self`, enabling method chaining for more fluent interfaces.
  - Consistency: It provides a consistent way to refer to the instance across all methods in the class.
  - Explicit self: Unlike some other languages, Python requires explicit declaration of `self` as the first parameter in instance methods, making the code more readable and self-documenting.

### 3. Explain the difference between attributes and methods in a class.

- Attributes:
    - **Data representation: Attributes are variables that store data associated with an instance or the class itself.**
    - State: They represent the state or properties of an object.
    - Access: Attributes are typically accessed using **dot notation** (e.g., `object.attribute`).
    - Types: Can be instance variables (unique to each instance) or class variables (shared among all instances).
    - Modification: Can be directly modified (unless protected by properties or private naming conventions).

- Methods:
    - **Behavior representation: Methods are functions that define the behaviors or actions an object can perform.**
    - Functionality: They encapsulate operations that can be performed on the object's data.
    - Invocation: Methods are **called using parentheses** (e.g., `object.method()`).
    - Parameters: Can accept parameters and return values.
    - Access to attributes: Methods can access and modify the object's attributes.
    - Types: Can be instance methods, class methods, or static methods.

### 4. How do you create an instance of a class in Python?

Creating an instance of a class in Python involves the following steps:

- Class definition: Ensure the class is defined and available in the current scope.
- Constructor call: Use the class name followed by parentheses, as if calling a function. This invokes the `__init__()` method.
- Arguments: If the `__init__()` method requires arguments, pass them within the parentheses.
- Assignment: Typically, you assign the new instance to a variable for later use.

**Example:**

```python
class MyClass:
    def __init__(self, param1, param2):
        self.attr1 = param1
        self.attr2 = param2

# Creating an instance
var_name = MyClass("value1", "value2")
```

In this process:

- The class name `MyClass` is called like a function.
- The `__init__()` method is automatically invoked with the provided arguments.
- A new instance is created and initialized.
- The instance is assigned to the variable `var_name`.


### 5. Describe the difference between `__str__()` and `__repr__()` methods.


- `__str__()`
    - Intended for end-users: Provides a concise, readable string representation of the object.
    - Used by `str()` and `print()`: Called by these functions to get a string version of the object.
    - Informal representation: Focuses on clarity and readability over completeness.
    - Can omit details: May not include all information about the object's state.

- `__repr__()`:
    - Intended for developers: Provides a detailed, unambiguous representation of the object.
    - Used in interactive environments: Called in the Python shell when evaluating expressions.
    - Formal representation: Aims to provide a complete description of the object.
    - Should be unambiguous: Ideally, `eval(repr(obj))` should recreate the object.
    - Fallback for `__str__()`: If `__str__()` is not defined, Python falls back to `__repr__()`.
    - **Used in printing collections**: When printing a list of objects, Python uses `__repr__()` to get a string representation of each object.

**Example:**

```python
class Person:
    def __init__(self, name, age):
        self.name = name
        self.age = age
    def __str__(self):
        return f"{self.name}, {self.age} years old"
    def __repr__(self):
        return f"Person(name='{self.name}', age={self.age})"

p = Person("Alice", 30)
print(str(p)) # Output: Alice, 30 years old
print(repr(p)) # Output: Person(name='Alice', age=30)
```

### 10. Why is encapsulation important in object-oriented programming?

- Data hiding: It restricts direct access to some of an object's components, protecting the internal state of an object from unauthorized access or modification.
- **Modularity:** It bundles data and methods that operate on that data within a single unit (the class), improving code organization and readability.
- Flexibility and maintenance: It allows changing the internal implementation without affecting the external code that uses the class, making the system more flexible and easier to maintain.
- Reduced complexity: By hiding the internal details, it reduces the complexity of the code from an external perspective.
- Improved testability: Encapsulated code is often easier to test as it provides a clear interface for interaction.
- Data integrity: It helps maintain the integrity of the data by controlling how it can be accessed and modified.
- **Abstraction:** It supports the concept of abstraction by hiding unnecessary details and exposing only the essential features of an object.
- Security: It can prevent unauthorized access to internal data, improving the security of the system.
- Code reusability: Encapsulated code is often more reusable as it's self-contained and has clear boundaries.
- Reduced dependencies: It minimizes the interdependencies between different parts of the code, leading to more robust and maintainable software.
