import argparse

def ex1():
    def add(a, b):
        return a + b

    result1 = add(10, 20)  # Works fine; returns 30
    print(result1)
    result2 = add("Hello, ", "World!")  # Works fine; returns "Hello, World!"
    print(result2)
    result3 = add("Hello, ", 20)  # Raises a TypeError
    print(result3)


def ex2():
    def add(a: int, b: int) -> int:
        return a + b

    result1 = add(10, 20)  # Works fine; returns 30
    result2 = add("Hello, ", "World!")  # Works fine (Even with the str type); returns "Hello, World!"
    result3 = add("Hello, ", 20)  # Raises a TypeError

    print(f"Typed add results: {result1}, {result2}")
    # Note: result3 will raise an error, so we don't print it


def ex3():
    def add(a, b):
        if not (isinstance(a, int) and isinstance(b, int)):
            raise TypeError("Both arguments must be of type int")
        return a + b

    result1 = add(10, 20)  # Works fine; returns 30

    try:
        result2 = add("Hello, ", "World!")  # Raises a TypeError
    except TypeError as e:
        print(f"TypeError occurred: {e}")

    try:
        result3 = add("Hello, ", 20)  # Raises a TypeError
    except TypeError as e:
        print(f"TypeError occurred: {e}")

    print(f"Strict add result: {result1}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Section A Examples")
    parser.add_argument("function", choices=["ex1", "ex2", "ex3"], help="Choose which add function to run")
    args = parser.parse_args()

    if args.function == "ex1":
        ex1()
    elif args.function == "ex2":
        ex2()
    elif args.function == "ex3":
        ex3()