"""
B. Invocation vs Inspection
    - Invocation: calling a method on an object
        - obj.method()
    - Inspection: External code that examines an object
        - isinstance(obj, type)
        - issubclass(subclass, type)
        - dir(obj)
        - inspect.getmembers(obj)
        - inspect.getsource(obj)
        - inspect.getmodule(obj)
        - inspect.getfile(obj)
        - inspect.getsourcefile(obj)
        - inspect.getsourcelines(obj)
        - inspect.getargvalues(obj)


















"""

import argparse
import inspect
from pprint import pformat
import io
import sys

class myObject:
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return f"MyObject({self.value})"

    def __repr__(self):
        return f"MyObject({self.value})"

    def __format__(self, format_spec):
        return f"MyObject({self.value})"

    def my_getattr(self, attr):
        return f"MyObject({self.value})"

    def my_setattr(self, attr, value):
        self.value = value


def ex1():
    def inspect_object(obj):
        print(f"Object type: {type(obj)}")
        print(f"Object attributes: {pformat(dir(obj))}")

        # Capture help() output
        old_stdout = sys.stdout
        help_output = io.StringIO()
        sys.stdout = help_output
        help(obj)
        sys.stdout = old_stdout
        print("Help output:")
        print(help_output.getvalue())

        # Only try to get source code for supported types
        if inspect.ismodule(obj) or inspect.isclass(obj) or inspect.ismethod(obj) or inspect.isfunction(obj) or inspect.iscode(obj):
            try:
                print(f"Object source code: {inspect.getsource(obj)}")
            except TypeError:
                print("Source code not available for this object type")
        else:
            print("Source code not available for this object type")

    inspect_object(myObject)
    # inspect_object(pformat)
    # inspect_object("Hello")
    # inspect_object([1, 2, 3])
    # inspect_object({"a": 1, "b": 2})



def ex2():
    pass


def ex3():
    pass


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Section B Examples")
    parser.add_argument("function", choices=["ex1", "ex2", "ex3"], help="Choose which add function to run")
    args = parser.parse_args()

    if args.function == "ex1":
        ex1()
    elif args.function == "ex2":
        ex2()
    elif args.function == "ex3":
        ex3()