""" Simple Class Definition and Usage """

class Person: # Class Definition

    def __init__(self, name, age): # Constructor
        # Attributes
        self.name = name
        self.age = age

    # __str__() for end-users (called by str() and print())
    def __str__(self): # Special Method
        return f"{self.name}, {self.age} years old"

    # __repr__() for developers (called by repr() and interactive shell)
    # When printing a list of objects, Python uses __repr__() to get a string representation of each object
    def __repr__(self): # Special Method
        return f"Person(name={self.name}, age={self.age})"

    def __eq__(self, other): # Special Method
        return self.name == other.name and self.age == other.age

    def set_age(self, age): # Method
        self.age = age

new_person = Person("John", 20)
print(f"new_person: {new_person}")

# new_person is automatically passed as the first argument (self)
print(f"new_person.age: {new_person.age}")
new_person.set_age(21)
print(f"new_person.age: {new_person.age}")

# Calls the class method directly with the instance as the first argument passed to self parameter
# Helpful to understand what is happening when calling methods on instances of a class
Person.set_age(new_person, 22)
