""" Class Method Example """

# Example 1
class Date:
    def __init__(self, day: int, month: int, year: int) -> None:
        self.day: int = day
        self.month: int = month
        self.year: int = year

    @classmethod
    def from_string(cls, date_string):
        """
        Create a Date object from a string.
        Acceptable formats:
        - 'DD-MM-YYYY'
        - 'DD/MM/YYYY'
        - 'YYYY-MM-DD'
        """
        if '-' in date_string:
            parts = date_string.split('-')
            if len(parts[0]) == 4:  # YYYY-MM-DD
                year, month, day = map(int, parts)
            else:  # DD-MM-YYYY
                day, month, year = map(int, parts)
        elif '/' in date_string:
            day, month, year = map(int, date_string.split('/'))
        else:
            raise ValueError("Invalid date format. Use 'DD-MM-YYYY', 'DD/MM/YYYY', or 'YYYY-MM-DD'.")

        return cls(day, month, year)

    @classmethod
    def from_timestamp(cls, timestamp):
        """
        Create a Date object from a Unix timestamp.
        Unix timestamp format: 1714857600

        """
        import datetime
        date = datetime.datetime.fromtimestamp(timestamp)
        return cls(date.day, date.month, date.year)

    @classmethod
    def today(cls):
        """Create a Date object representing today's date.

        Example:
        date = Date.today()
        print(date) # Output: Date(day=23, month=6, year=2023)
        """
        import datetime
        today = datetime.date.today()
        return cls(today.day, today.month, today.year)

# Using the class method
date = Date.from_string('23-06-2023')


# =================================================================== #
# Example 2
class MathUtils:
    PI = 3.14159

    @classmethod
    def circle_area(cls, radius):
        return cls.PI * radius ** 2

    @classmethod
    def circle_circumference(cls, radius):
        return 2 * cls.PI * radius

# Using the class without instantiation
area = MathUtils.circle_area(5)
circumference = MathUtils.circle_circumference(5)

print(f"Area: {area}, Circumference: {circumference}")