import inspect
import os
import io
import sys

# Code to test if Jinja2 is installed and if not, install it
try:
    from jinja2 import Environment
except ImportError:
    print("Jinja2 is not installed. Installing...")
    os.system("pip install Jinja2")


help_template = """

{{ magenta }}# ============================== help(person) ===================================== #
Help on {{ red }}Person{{ magenta }} in module __main__ object:

{{ red }}class Person(builtins.object)
 |  Person({{ yellow }}first_name, last_name, age, email{{ red }})
 |
 |  {{ green }}A class representing a person.{{ red }}
 |
 {{ blue }}|  Methods defined here:{{ cyan }}
 |
 {{ cyan }}|  __init__({{ red }}self,{{ yellow }} first_name, last_name, age, email{{ cyan }})
 |      {{ green }}Initialize a new Person instance.{{ cyan }}
 |
 |  __repr__({{ red }}self{{ cyan }})
 |      {{ green }}Return a detailed string representation of the person.{{ cyan }}
 |
 |  __str__({{ red }}self{{ cyan }})
 |      {{ green }}Return a string representation of the person.{{ cyan }}
 |
 |  {{ yellow }}greet({{ red }}self,{{ yellow }} name){{ cyan }}
 |      {{ green }}Return a greeting message.{{ cyan }}
 |
 |  ----------------------------------------------------------------------
 |  {{ blue }}Data descriptors defined here:{{ cyan }}
 |
 |  __dict____
 |      {{ green }}dictionary for instance variables{{ cyan }}
 |
 |  {{ cyan }}__weakref____
 |      {{ green }}list of weak references to the object{{ cyan }}

{{ magenta }}# =================================== help(list) ======================================= #
{{ magenta }}Help on class {{ red }}list{{ magenta }} in module builtins:

{{ red }}class list(object)
 |  list(iterable=(), /)
 |
 |  {{ green }}Built-in mutable sequence.{{ red }}
 |
 |  {{ green }}If no argument is given, the constructor creates a new empty list.{{ red }}
 |  {{ green }}The argument must be an iterable if specified.{{ red }}
 |
 {{ blue }}|  Methods defined here:{{ cyan }}
 |
 |  __add__({{ red }}self{{ cyan }}, value, /)
 |      {{ green }}Return self+value.{{ cyan }}
 |
 |  __contains__({{ red }}self{{ cyan }}, key, /)
 |      {{ green }}Return bool(key in self).{{ cyan }}
 |
 |  __eq__({{ red }}self{{ cyan }}, value, /)
 |      {{ green }}Return self==value.{{ cyan }}
 |
 |  __getattribute__({{ red }}self{{ cyan }}, name, /)
 |      {{ green }}Return getattr(self, name).{{ cyan }}
 |
 |  __getitem__({{ red }}self{{ cyan }}, index, /)
 |      {{ green }}Return self[index].{{ cyan }}
 |
 |  __init__({{ red }}self{{ cyan }}, /, *args, **kwargs)
 |      {{ green }}Initialize self.  See help(type(self)) for accurate signature.{{ cyan }}
 |
 |  __iter__({{ red }}self{{ cyan }}, /)
 |      {{ green }}Implement iter(self).{{ cyan }}
 |
 |  __len__({{ red }}self{{ cyan }}, /)
 |      {{ green }}Return len(self).{{ cyan }}
 |
 |  __repr__({{ red }}self{{ cyan }}, /)
 |      {{ green }}Return repr(self).{{ cyan }}
 |
 |  __setitem__({{ red }}self{{ cyan }}, key, value, /)
 |      {{ green }}Set self[key] to value.{{ cyan }}
 |
 |  __sizeof__({{ red }}self{{ cyan }}, /)
 |      {{ green }}Return the size of the list in memory, in bytes.{{ cyan }}
 |
 |  append({{ red }}self{{ cyan }}, object, /)
 |      {{ green }}Append object to the end of the list.{{ cyan }}
 |
 |  index({{ red }}self{{ cyan }}, value, start=0, stop=9223372036854775807, /)
 |      {{ green }}Return first index of value.{{ cyan }}
 |
 |      {{ green }}Raises ValueError if the value is not present.{{ cyan }}
 |
 |  insert({{ red }}self{{ cyan }}, index, object, /)
 |      {{ green }}Insert object before index.{{ cyan }}
 |
 |  pop({{ red }}self{{ cyan }}, index=-1, /)
 |      {{ green }}Remove and return item at index (default last).{{ cyan }}
 |
 |      {{ green }}Raises IndexError if list is empty or index is out of range.{{ cyan }}
 |
 |  remove({{ red }}self{{ cyan }}, value, /)
 |      {{ green }}Remove first occurrence of value.{{ cyan }}
 |
 |      {{ green }}Raises ValueError if the value is not present.{{ cyan }}
 |
 |  ----------------------------------------------------------------------
 |  {{ blue }}Class methods defined here:{{ cyan }}
 |
 |  {{ cyan }}__class_getitem__(...)
 |      {{ green }}See PEP 585{{ cyan }}
 |
 |  ----------------------------------------------------------------------
 |  {{ blue }}Static methods defined here:{{ cyan }}
 |
 |  {{ cyan }}__new__(*args, **kwargs)
 |      {{ green }}Create and return a new object.  See help(type) for accurate signature.{{ cyan }}
 |
"""

# ================================================================================================== #

# =================== #
# Utility Functions
# =================== #

# Jinja2 setup
env = Environment()

def render_colored_template(template_string, **kwargs):
    """Render a colored template using Jinja2."""
    template = env.from_string(template_string)

    color_vars = {
        'red': '\033[91m',
        'green': '\033[92m',
        'yellow': '\033[93m',
        'blue': '\033[94m',
        'magenta': '\033[95m',
        'cyan': '\033[96m',
        'reset': '\033[0m'
    }
    rendered_content = template.render(**kwargs, **color_vars)
    print(rendered_content)

# Colorize text with a given color
def colorize(text, color):
    """Colorize text with a given color."""
    colors = {
        "red": "\033[91m",
        "green": "\033[92m",
        "yellow": "\033[93m",
        "blue": "\033[94m",
        "magenta": "\033[95m",
        "cyan": "\033[96m",
    }
    return f"{colors[color]}{text}\033[0m"

# Print the help information for an object without using a pager.
# def print_help(obj):
#     """Print the help information for an object without using a pager."""
#     old_stdout = sys.stdout
#     sys.stdout = io.StringIO()
#     help(obj)
#     output = sys.stdout.getvalue()
#     sys.stdout = old_stdout
#     print(colorize(f"Help for {obj.__class__.__name__}:", "magenta"))
#     print(colorize(output, "cyan"))

# Print a colorized description and execute a function.
def print_example(description, func, *args, **kwargs):
    """Print a colorized description and execute a function."""
    print(colorize(f"\n{description}:", "magenta"))
    result = func(*args, **kwargs)
    if result is not None:
        print(colorize(str(result), "cyan"))

def print_reference():
    """Print the Person class and object definitions for reference."""
    print(colorize("\nReference:", "magenta"))
    print(colorize("==========", "magenta"))
    print(colorize("""
""", "cyan") + colorize("class Person:", "red") + colorize("""
    """ + colorize("def __init__(self, first_name, last_name, age, email):", "green") + """
        self.first_name = first_name
        self.last_name = last_name
        self.age = age
        self.email = email

    """ + colorize("def greet(self, name):", "green") + """
        return f"Hello {name}, my name is {self.first_name} {self.last_name}."

    """ + colorize("def __str__(self):", "green") + """
        return f"{self.first_name} {self.last_name}, Age: {self.age}"

    """ + colorize("def __repr__(self):", "green") + """
        return (f"Person(first_name={self.first_name!r}, "
                f"last_name={self.last_name!r}, age={self.age}, email={self.email!r})")

""", "cyan") + colorize("john = ", "blue") + colorize("Person(", "red") + colorize('"John", "Doe", 30, "john.doe@example.com"', "yellow") + colorize(")", "red") + """
""" + colorize("jane = ", "blue") + colorize("Person(", "red") + colorize('"Jane", "Smith", 25, "jane.smith@example.com"', "yellow") + colorize(")", "red") + """
""" + colorize("alice = ", "blue") + colorize("Person(", "red") + colorize('"Alice", "Johnson", 28, "alice.johnson@example.com"', "yellow") + colorize(")", "red") + """
""" + colorize("bob = ", "blue") + colorize("Person(", "red") + colorize('"Bob", "Brown", 35, "bob.brown@example.com"', "yellow") + colorize(")", "red") + """
    """)

def print_menu():
    """Print the interactive menu."""
    print(colorize("\nPython Object Introspection Examples", "magenta"))
    print(colorize("====================================", "magenta"))
    for key, (name, _) in examples.items():
        print(colorize(f"{key}. {name}", "magenta"))
    print(colorize("0. Exit", "red"))

def clear_screen():
    """Clear the terminal screen."""
    os.system('cls' if os.name == 'nt' else 'clear')

# =================== #
# Reference Code
# =================== #

class Person:
    """A class representing a person."""

    def __init__(self, first_name, last_name, age, email):
        """Initialize a new Person instance."""
        self.first_name = first_name
        self.last_name = last_name
        self.age = age
        self.email = email

    def greet(self, name):
        """Return a greeting message."""
        return f"Hello {name}, my name is {self.first_name} {self.last_name}."

    def __str__(self):
        """Return a string representation of the person."""
        return f"{self.first_name} {self.last_name}, Age: {self.age}"

    def __repr__(self): # !r adds the quotes to the formatted string
        """Return a detailed string representation of the person."""
        return (f"Person(first_name={self.first_name!r}, "
                f"last_name={self.last_name!r}, age={self.age}, email={self.email!r})")

john = Person("John", "Doe", 30, "john.doe@example.com")
jane = Person("Jane", "Smith", 25, "jane.smith@example.com")
alice = Person("Alice", "Johnson", 28, "alice.johnson@example.com")
bob = Person("Bob", "Brown", 35, "bob.brown@example.com")

# =================== #
# Main Logic
# =================== #


class IntrospectionExample:
    @staticmethod
    def basic_introspection(person=john):
        """Examples of basic introspection functions."""
        print_example("dir(person)", dir, person)
        print_example("vars(person)", vars, person)
        # print_example("print(person)", print, person)
        print(colorize("\nprint(person):", "magenta"))
        print(colorize(f"{person}", "cyan"))
        print_example("repr(person)", repr, person)

    @staticmethod
    def attribute_introspection(person=jane):
        """Examples of attribute introspection functions."""
        print_example("getattr(person, 'first_name')", getattr, person, 'first_name')
        print_example("hasattr(person, 'age')", hasattr, person, 'age')
        print(colorize("\nBefore setattr:", "magenta"))
        print(colorize(f"person.age = {person.age}", "cyan"))
        setattr(person, 'age', 26)
        print(colorize("After setattr(person, 'age', 26):", "magenta"))
        print(colorize(f"person.age = {person.age}", "cyan"))

    @staticmethod
    def type_introspection(person=alice):
        """Examples of type introspection functions."""
        print_example("type(person)", type, person)
        print_example("isinstance(person, Person)", isinstance, person, Person)
        print_example("issubclass(Person, object)", issubclass, Person, object)

    @staticmethod
    def help_example(person=bob):
        """Examples of callable() and help() functions."""
        render_colored_template(help_template)

    @staticmethod
    def inspect_examples(person=john):
        """Examples using the inspect module."""
        # print_example("inspect.getmembers(person)", inspect.getmembers, person)
        print_example("inspect.getsource(Person)", inspect.getsource, Person)
        print_example("inspect.signature(Person)", inspect.signature, Person)
        print_example("inspect.ismethod(person.greet)", inspect.ismethod, person.greet)

# Dictionary mapping menu options to functions
examples = {
    '1': ('Basic Introspection', IntrospectionExample.basic_introspection),
    '2': ('Attribute Introspection', IntrospectionExample.attribute_introspection),
    '3': ('Type Introspection', IntrospectionExample.type_introspection),
    '4': ('Callable and Help Example', IntrospectionExample.help_example),
    '5': ('Inspect Module Examples', IntrospectionExample.inspect_examples),
}

def main():
    """Main function to run the interactive menu."""
    while True:
        clear_screen()
        print_reference()
        print_menu()
        choice = input(colorize("\nEnter your choice (0-5): ", "green"))

        if choice == '0':
            print(colorize("Exiting the program. Goodbye!", "red"))
            sys.exit(0)

        if choice in examples:
            name, func = examples[choice]
            print(colorize(f"\nRunning: {name}", "blue"))
            func()
            input(colorize("\nPress Enter to continue...", "green"))
            clear_screen()
        else:
            clear_screen()

if __name__ == "__main__":
    main()