from names_module import *

# Module-level tests
print(public_var)  # Works: "I'm a public module variable"
print(public_function())  # Works: "I'm a public module function"

# These will raise NameError because they're not imported with *

try:
    print(_private_var)
except NameError:
    print("NameError: name '_private_var' is not defined")
try:
    print(_private_function())
except NameError:
    print("NameError: name '_private_function' is not defined")
try:
    print(__mangled_var)
except NameError:
    print("NameError: name '__mangled_var' is not defined")

# Class tests
obj = MyClass()
child_obj = ChildClass()

print(obj.public)  # Works: 3
print(obj.public_method())  # Works: "I'm a public method"

# These work, but IDEs might warn about accessing protected members
print(obj._weak_private)  # 1
print(obj._private_method())  # "I'm a private method"

# This raises AttributeError
# print(obj.__strong_private)

# Accessing name-mangled attributes (strongly discouraged)
print(obj._MyClass__strong_private)  # 2
print(child_obj._MyClass__strong_private)  # 2 (parent's attribute)
print(child_obj._ChildClass__strong_private)  # 4 (child's own attribute)

# Accessing through a method (preferred way)
print(obj.get_strong_private())  # 2
print(child_obj.get_strong_private())  # 2 (uses parent's method)

# Demonstrating that ChildClass.__strong_private doesn't override MyClass.__strong_private
print(isinstance(child_obj, MyClass))  # True
print(child_obj.get_strong_private())  # Still 2, not 4

# Direct class attribute access (not recommended)
try:
    print(MyClass._MyClass__strong_private)  # Raises AttributeError (class attribute doesn't exist)
except AttributeError:
    print("AttributeError: 'MyClass' object has no attribute '_MyClass__strong_private'")
try:
    print(ChildClass._ChildClass__strong_private)  # Raises AttributeError (class attribute doesn't exist)
except AttributeError:
    print("AttributeError: 'ChildClass' object has no attribute '_ChildClass__strong_private'")

# Modifying private attributes (possible, but discouraged)
obj._weak_private = 10
print(obj._weak_private)  # 10

obj._MyClass__strong_private = 20
print(obj.get_strong_private())  # 20