from dataclasses import dataclass

@dataclass # automatically generates __init__, __repr__, __eq__
class Product1:
    name: str
    price: float
    quantity: int = 0

    def total_price(self) -> float:
        return self.price * self.quantity


p1 = Product1("Laptop1", 1000, 1)
p2 = Product1("Laptop2", 750, 2)

print(f"p1: {p1.total_price()}")
print(f"p2: {p2.total_price()}")
print(f"Product1:\n{dir(Product1)}")

try:
    print(f"is p1 == p2: {p1 == p2}")  # False
except TypeError as e:
    print(f"Error comparing equality: {e}")

try:
    print(f"is p1 > p2: {p1 > p2}")  # True
except TypeError as e:
    print(f"Error comparing greater than: {e}")

try:
    print(f"is p1 < p2: {p1 < p2}")  # False
except TypeError as e:
    print(f"Error comparing less than: {e}")

@dataclass(order=True) # order=True allows us to compare two dataclasses
class Product2:
    name: str
    price: float
    quantity: int = 0

    def total_price(self) -> float:
        return self.price * self.quantity


p3 = Product2("Laptop", 1200, 1)
p4 = Product2("Laptop", 1200, 1)
p5 = Product2("tablet", 250.99, 4)

print("\n\n\n")
print(f"Product2:\n{dir(Product2)}")

print(f"p3: {p3}")
print(f"p4: {p4}")
print(f"p5: {p5}")

print(f"is p3 == p4: {p3 == p4}")  # True
print(f"is p3 > p5: {p3 > p5}")  # True
print(f"is p3 < p5: {p5 < p3}")  # True
