import functools
import logging
import os
import statistics

# Configure logging
logging.basicConfig(level=logging.DEBUG)

# Global flag for enabling/disabling logging
ENABLE_LOGGING = os.environ.get("ENABLE_LOGGING", "False").lower() == "true"


def log_function(func):
    """Log the function signature and return value if logging is enabled"""

    @functools.wraps(func)
    def wrapper_log(*args, **kwargs):
        if ENABLE_LOGGING:
            # Create a list of positional arguments
            # using repr() to get the string representation of each argument
            args_repr = [repr(a) for a in args]
            print(f"args_repr: {args_repr}")
            # Create a list of keyword arguments
            # using an f-string to format each key-value pair
            # and repr() to get the string representation of each value
            kwargs_repr = [f"{k}={repr(v)}" for k, v in kwargs.items()]
            print(f"kwargs_repr: {kwargs_repr}")
            # Combine the positional and keyword arguments into a single string
            signature = ", ".join(args_repr + kwargs_repr)
            # Log the function call with the signature
            logging.debug(f"Calling {func.__name__}({signature})")
            value = func(*args, **kwargs)
            logging.debug(f"{func.__name__}() returned {repr(value)}")
            return value
        return func(*args, **kwargs)

    return wrapper_log


@log_function
def calculate_stats(*numbers, operation="mean", round_to=2):
    """Calculate statistics for given numbers"""
    if not numbers:
        return None

    result = None
    if operation == "mean":
        result = statistics.mean(numbers)
    elif operation == "median":
        result = statistics.median(numbers)
    elif operation == "sum":
        result = sum(numbers)
    else:
        raise ValueError(f"Unsupported operation: {operation}")

    return round(result, round_to)


# Example usage
print(calculate_stats(1, 2, 3, 4, 5))
print(calculate_stats(1, 2, 3, 4, 5, operation="median"))
print(calculate_stats(10, 20, 30, operation="sum", round_to=0))
