"""
This module showcases automatic class registration using metaclasses in Python.

Automatic registration is useful for creating plugin systems or extensible architectures
where new classes can be easily added and discovered without modifying existing code.

The module defines a PluginRegistry metaclass that automatically registers subclasses
of BasePlugin. This allows for easy discovery and management of plugin classes.
"""

class PluginRegistry(type):
    plugins = {}

    def __new__(cls, name, bases, attrs):
        new_cls = type.__new__(cls, name, bases, attrs)
        if name != 'BasePlugin':
            # Register the new class if it's not the base class
            cls.plugins[name] = new_cls
        return new_cls

class BasePlugin(metaclass=PluginRegistry):
    pass

# Usage
class MyPlugin(BasePlugin):
    def run(self):
        print("MyPlugin is running")

print(PluginRegistry.plugins)  # Output: {'MyPlugin': <class '__main__.MyPlugin'>}
# This shows that MyPlugin was automatically registered