"""
This module demonstrates the Singleton pattern implementation using metaclasses in Python.

The Singleton pattern ensures that a class has only one instance and provides a global point
of access to that instance. This is useful for managing shared resources or maintaining a
single state across the application.

The module defines a Singleton metaclass and a DatabaseConnection class that uses this
metaclass to ensure only one connection is created.
"""

# Singleton Pattern Example
class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            # Create a new instance if it doesn't exist
            cls._instances[cls] = super().__call__(*args, **kwargs)
        return cls._instances[cls]

class DatabaseConnection(metaclass=Singleton):
    def __init__(self):
        self.connection = "Connected to database"

# Usage
db1 = DatabaseConnection()
db2 = DatabaseConnection()
print(db1 is db2)  # Output: True
# This demonstrates that db1 and db2 refer to the same instance