
class SimpleIterable:
    def __init__(self, *args):
        self.args = args

    def __iter__(self):
        return SimpleIterator(self.args)

class SimpleIterator:
    def __init__(self, args):
        self.args = args
        self.index = 0

    def __iter__(self):
        return self

    def __next__(self):
        if self.index >= len(self.args):
            raise StopIteration
        value = self.args[self.index]
        self.index += 1
        return value

# Usage demonstration
simple = SimpleIterable(10, 20, 30, 40)

# Iterate over all arguments
print("Iterating over all items:")
for item in simple:
    print(item)

# Create an iterator and use next()
print("\nUsing next():")
iterator = iter(simple)
print(next(iterator))  # 10
print(next(iterator))  # 20

# Demonstrate that we can create multiple independent iterators
print("\nMultiple iterators:")
iter1 = iter(simple)
iter2 = iter(simple)
print(f"iter1: {next(iter1)}, {next(iter1)}")  # 10, 20
print(f"iter2: {next(iter2)}")  # 10 (starts from beginning)

# Show that the original object is not an iterator
print("\nOriginal object is not an iterator:")
try:
    next(simple)
except TypeError as e:
    print(f"Error: {e}")

# Demonstrate flexibility with different numbers of arguments
print("\nFlexibility with arguments:")
simple_two = SimpleIterable("a", "b")
simple_many = SimpleIterable(*range(10))

print("Two args:", list(simple_two))
print("Many args:", list(simple_many))