# 170D Python Course

## Python 1 Block 1

### Day 1

- Programming Fundamentals
    - What is Programming?
    - Sequential Instruction
    - Programming Languages
    - Compiled vs Interpreted Languages
    - Code and Data
    - Basic Syntax Concepts
- Python Ecosystem
    - Python Installation and Versions
    - Python Execution Methods
        - Command Line
        - Interactive Shell (REPL)
    - Python Documentation
        - `help()` and `dir()`
    - The Standard Library
    - PyPI and Third-Party Packages
    - Virtual Environments (venv)
    - Package Management (pip)
    - Terminology

### Day 2

- Python Basics
    - Python Style Guide and PEPs
    - Variables and Assignment
    - Dynamic vs Static Typing
    - Basic Data Types Overview
    - Strings (Basic)
    - Type Conversion
    - Operators
    - Python Keywords

### Day 3

- Strings and IO
    - String Operations
    - String Methods
    - String Formatting
        - f-strings
        - str.format()
        - %-formatting
    - Input/Output
        - print()
        - input()
    - Working with Text Files
        - Basic Reading/Writing
        - with statement introduction
        - Common file operations

### Day 4

- Data Structures
    - Lists
        - Creation and Access
        - List Methods
        - List Operations
    - Dictionaries
        - Key-Value Pairs
        - Dictionary Methods
        - Common Operations
### Day 5

- Control Flow
    - Conditional Statements
        - if/elif/else
        - Comparison Operators
        - Logical Operators
        - Ternary Operators
    - Loops
        - for loops
        - while loops
        - Loop Control (break, continue)
        - Loop with else
        - Comprehensions (Basic)

### Day 6

- Lab / Review Day

## Python 1 Block 2

### Day 1

- Advanced Data Concepts
    - Mutability vs Immutability
    - Variable References
    - Shallow vs Deep Copy
    - Memory Management Basics
    - Advanced Collections
        - Tuples
        - Sets
        - Named Collections

### Day 2

- Functions
    - Function Definition
    - Parameters and Arguments
    - Return Values
    - Scope and Namespaces
    - Lambda Functions
    - Function Documentation
    - Type Hints (Introduction)
    - Best Practices

### Day 3

- Error Handling
    - Exception Hierarchy
    - Try/Except/Else/Finally
    - Context Managers
    - Debugging Techniques


### Day 4

- File System Operations
    - Path Management
        - os.path
        - pathlib
    - File Operations
        - Reading/Writing
        - Binary Files
        - Context Managers
    - Directory Operations
    - File System Navigation
- Logging

### Day 5

- Nmap Scan Results - Practical

### Day 6

- Lab / Review Day

## Python 2 Block 1

### Day 1

- Modules and Packages
    - Module Basics
    - Package Structure
    - Import System
    - Creating Modules
    - Module Search Path
    - Package Distribution
    - Common Standard Library Modules

### Day 2

- Testing and Quality
    - Unit Testing
    - Test Driven Development
    - Code Quality Tools
    - Documentation
    - Type Checking
    - Code Coverage

### Day 3

- Advanced Functions
    - Decorators
    - Context Managers
    - Closures
    - Advanced Arguments
    - Function Annotations
    - Functional Programming Concepts

### Day 4

- System Interaction
    - OS Interface
    - Subprocess Management
    - Signal Handling
    - System Commands
    - Environment Management

### Day 5

- Advanced Topics
    - Advanced Collections
    - Date and Time Handling
    - Command Line Arguments
    - Environment Variables
    - Performance Optimization

### Day 6

- Lab / Review Day

## Python 2 Block 2

### Day 1

- Advanced Python Concepts
    - Iterators and Iterables
    - Generators
    - Decorators (Comprehensive Coverage)
    - Context Managers (Comprehensive Coverage)
    - Advanced Comprehensions

### Day 2

- Data Handling
    - Structured Data Processing
    - Serialization Formats
    - Data Validation
    - Data Transformation
    - Working with APIs

### Day 3

- Regular Expressions
    - Pattern Syntax
        - Basic Patterns
        - Character Classes
        - Quantifiers
        - Groups and Capturing
        - Anchors and Boundaries
    - re Module
        - Pattern Objects
        - Match Objects
        - Common Methods (search, match, findall, finditer)
        - Split and Replace Operations
    - Pattern Compilation and Flags
    - Common Use Cases
        - Text Validation
        - Data Extraction
        - String Parsing
        - Log File Analysis
    - Best Practices and Optimization

### Day 4

- Introduction to Classes
    - Classes and Objects Basics
    - Class Attributes vs Instance Attributes
    - Encapsulation
    - Inheritance
    - Abstraction
    - Polymorphism

### Day 5

- Logging API
    - Logging Levels
    - Logging Formats
    - Logging to different outputs
- Development Tools
    - Debugging Tools
    - Profiling
    - Documentation Tools
    - Static Analysis
    - Code Quality Tools


### Day 6

- Lab / Review Day

## Python OOP Block

### Day 1

- Object-Oriented Programming Fundamentals
    - Classes and Objects Basics
    - Instance Creation and Initialization
    - Instance vs Class Attributes
    - Methods and Self
    - Access Control and Properties
    - Class Methods and Static Methods
    - Method Types in Practice

### Day 2

- Python Data Model Fundamentals
    - Special Methods Overview
    - Object Representation (`__repr__`, `__str__`)
    - Collections and Containers (`__len__`, `__getitem__`, `__setitem__`)
    - Attribute Access (`__getattr__`, `__setattr__`)
    - Callable Objects (`__call__`)
    - Context Managers (`__enter__`, `__exit__`)
    - Custom Sequence Types

### Day 3

- Inheritance and Composition
    - Custom Exceptions
    - Inheritance Fundamentals
    - Method Resolution Order (MRO)
    - Super() and Method Delegation
    - Multiple Inheritance
    - Composition vs Inheritance
    - Mixin Classes
    - Abstract Base Classes

### Day 4

- Advanced Data Model Concepts
    - Object Creation Control (`__new__`, `__init__`)
    - Attribute Management
    - Descriptors and Properties
    - Managed Attributes
    - Slots and Memory Optimization
    - Object Lifecycle (`__del__`)

### Day 5
- Testing Object-Oriented Code
    - Unit Testing Classes
    - Test Doubles (Mocks, Stubs)
    - Testing Inheritance
    - Testing Protocols
    - Integration Testing
    - Test-Driven Development

### Day 6
- SOLID Design Principles
    - Single Responsibility Principle
    - Open/Closed Principle
    - Liskov Substitution Principle
    - Interface Segregation Principle
    - Dependency Inversion Principle
    - Practical Applications

### Day 7

- Design Patterns
    - Creational Patterns
        - Factory Method
        - Singleton
        - Builder
    - Structural Patterns
        - Adapter
        - Decorator
        - Composite
    - Behavioral Patterns
        - Observer
        - Strategy
        - Command

### Day 8

- Advanced Class Features
    - Metaclasses Introduction
    - Class Decorators
    - Dynamic Attribute Creation
    - Runtime Class Modification
    - Advanced Property Patterns
    - Operator Overloading

### Day 9

- Object Protocols and Duck Typing
    - Sequence Protocol
    - Iterator Protocol
    - Container Protocol
    - Context Manager Protocol
    - Callable Protocol
    - Protocol Design Patterns

### Day 10

- Advanced OOP Patterns
    - Event-Driven Programming
    - Publish-Subscribe Pattern
    - Model-View-Controller
    - State Machines
    - Object Pools
    - Plugin Systems

### Day 11

- Lab / Review Day

