"""Mutable vs Immutable Objects in Python"""
# pylint: disable=W0105

from rich.console import Console
from rich.syntax import Syntax
from icecream import ic
import sys
import os

try:
    # add the directory above the current directory to the Python path
    sys.path.append(
        os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    )
    import src.instructor_tools as it
except ImportError:
    print("Error: instructor_tools not found")
    exit(1)

it.clear_screen()
c = Console()


# -----------------------------------------------------------------------
# Mutability Example (Lists)
# -----------------------------------------------------------------------
def ex1():
    """Reference Code:
    my_list = [1, 2, 3]
    another_list = my_list
    my_list[0] = 100
    """

    c.rule(
        title="[red]Mutability Example (Lists)[/red]",
        style="bold green",
        align="center",
    )

    it.print_comment("# Create a list with 3 elements")
    code = Syntax(
        ">>> my_list = [1, 2, 3]",
        "python",
        line_numbers=True,
    )
    c.print(code)
    my_list = [1, 2, 3]
    ic(my_list, id(my_list))
    c.input(">>>")

    it.print_comment("# Assign a new name to the original list object")
    code = Syntax(
        ">>> another_list = my_list",
        "python",
        line_numbers=True,
    )
    c.print(code)
    another_list = my_list
    ic(another_list, id(another_list))
    c.input(">>>")

    it.print_comment("# Modify the original list")
    code = Syntax(
        ">>> my_list[0] = 100",
        "python",
        line_numbers=True,
    )
    c.print(code)
    my_list[0] = 100
    ic(my_list, id(my_list))
    ic(another_list, id(another_list))
    c.input(">>>")


# -----------------------------------------------------------------------
# Immutable Example (String)
# -----------------------------------------------------------------------
def ex2():
    """Reference Code:
    my_string = "hello"
    another_string = my_string
    my_string = "hello world"
    another_string = another_string + " world"
    """

    c.rule(
        title="[red]Immutable Example (String)[/red]",
        style="bold green",
        align="center",
    )

    it.print_comment("# Create a string")
    code = Syntax(
        ">>> my_string = 'hello'",
        "python",
        line_numbers=True,
    )
    c.print(code)
    my_string = "hello"
    ic(my_string, id(my_string))
    c.input(">>>")

    it.print_comment("# Assign a new name to the original string object")
    code = Syntax(
        ">>> another_string = my_string",
        "python",
        line_numbers=True,
    )
    c.print(code)
    another_string = my_string
    ic(another_string, id(another_string))
    c.input(">>>")

    it.print_comment("# Modify the original string")
    code = Syntax(
        ">>> my_string = 'hello world'",
        "python",
        line_numbers=True,
    )
    c.print(code)
    my_string = "hello world"
    ic(my_string, id(my_string))
    ic(another_string, id(another_string))
    c.input(">>>")

    it.print_comment("# Modify the second string")
    code = Syntax(
        ">>> another_string = another_string + ' world'",
        "python",
        line_numbers=True,
    )
    c.print(code)
    another_string = another_string + " world"
    ic(my_string, id(my_string))
    ic(another_string, id(another_string))
    c.input(">>>")


# -----------------------------------------------------------------------
# Demonstrating function argument behavior
# -----------------------------------------------------------------------
def ex3():
    """Reference Code:
    def modify_list(a_list):
        a_list.append(5)

    test_list = [5, 2, 11]
    modify_list(test_list)

    sorted_1 = sorted(test_list)
    sorted_2 = test_list.sort()
    """

    c.rule(
        title="[red]Function Behavior[/red]",
        style="bold green",
        align="center",
    )

    function_code = """def modify_list(a_list):
        a_list.append(5)"""

    function_syntax = Syntax(
        function_code,
        "python",
        line_numbers=True,
    )
    it.print_comment("# Function Definition")
    c.print(function_syntax)

    def modify_list(a_list):
        a_list.append(5)

    it.print_comment("\n# Create a list")
    code = Syntax(
        ">>> test_list = [5, 2, 11]",
        "python",
        line_numbers=True,
    )
    c.print(code)
    test_list = [5, 2, 11]
    ic(test_list, id(test_list))
    c.input(">>>")

    it.print_comment("# Modify the list")
    code = Syntax(
        ">>> modify_list(test_list)",
        "python",
        line_numbers=True,
    )
    c.print(code)
    modify_list(test_list)
    ic(test_list, id(test_list))
    c.input(">>>")

    it.print_comment("# Sort the list in place")
    code = Syntax(
        ">>> test_list.sort()",
        "python",
        line_numbers=True,
    )
    c.print(code)
    test_list.sort()
    ic(test_list, id(test_list))
    c.input(">>>")

    it.print_comment("# Sort the list")
    code = Syntax(
        ">>> test_list = sorted(test_list)",
        "python",
        line_numbers=True,
    )
    c.print(code)
    test_list = sorted(test_list)
    ic(test_list, id(test_list))
    c.input(">>>")


if __name__ == "__main__":
    try:
        # ex1()
        # ex2()
        ex3()
    except KeyboardInterrupt:
        print("Exiting...")
